"""The fmu.settings.gui package."""

try:
    from ._version import __version__, version
except ImportError:
    __version__ = version = "0.0.0"

from .__main__ import run_server

__all__ = ["run_server"]



"""The main entry point for fmu-settings-gui."""

import asyncio
import os
import re
import signal
import sys
from pathlib import Path
from types import FrameType

import uvicorn
from fastapi import FastAPI, HTTPException
from fastapi.staticfiles import StaticFiles
from starlette.responses import FileResponse

app = FastAPI(title="FMU Settings GUI")

current_dir = Path(__file__).parent.absolute()
static_dir = current_dir / "static"


@app.get("/{full_path:path}")
async def serve_spa_catchall(full_path: str) -> FileResponse:
    """Ensures internal paths to the GUI are served by the SPA."""
    if full_path == "":
        full_path = "index.html"
    resolved_path = os.path.normpath(
        os.path.realpath(os.path.join(str(static_dir), full_path))
    )
    if bool(re.fullmatch(r"^[\w\s\.\-/]+$", str(resolved_path))) is False:
        raise ValueError(f"Unallowed characters present in {full_path!r}")
    if not resolved_path.startswith(str(static_dir)):
        raise HTTPException(status_code=403, detail="Access denied")
    if os.path.exists(resolved_path):
        return FileResponse(resolved_path)
    return FileResponse(static_dir / "index.html")


app.mount("/", StaticFiles(directory=str(static_dir), html=True), name="static")


def run_server(
    host: str = "127.0.0.1", port: int = 8000, log_level: str = "critical"
) -> None:
    """Starts the GUI server."""
    log_level = log_level.lower()

    server_config = uvicorn.Config(app=app, host=host, port=port, log_level=log_level)
    server = uvicorn.Server(server_config)

    def signal_handler(signum: int, frame: FrameType | None) -> None:
        """Gracefully handles interrupt shutdowns."""
        sys.exit(0)

    signal.signal(signal.SIGINT, signal_handler)
    signal.signal(signal.SIGTERM, signal_handler)

    try:
        asyncio.run(server.serve())
    except KeyboardInterrupt:
        sys.exit(0)


if __name__ == "__main__":
    run_server()



"""Builds the React app into the Python package."""

import subprocess
import sys
from pathlib import Path
from typing import Self

from setuptools import setup
from setuptools.command.build_py import build_py
from setuptools.command.develop import develop
from setuptools.command.sdist import sdist


def build_frontend() -> None:
    """Builds the frontend.

    Output from this function (print statements, output from pnpm) can only be viewed
    when the '-v' flag is given to 'pip install'.
    """
    root = Path(__file__).parent
    static = root / "src/fmu_settings_gui/static"

    frontend_dir = Path(root / "frontend")
    if not all((frontend_dir.exists(), frontend_dir.is_dir())):
        raise OSError("Frontend directory does not exist or is not a directory")

    try:
        print("Installing frontend packages...", file=sys.stderr)
        subprocess.run(["pnpm", "install"], cwd=frontend_dir, check=True)
        print("Building frontend...", file=sys.stderr)
        subprocess.run(["pnpm", "run", "build"], cwd=frontend_dir, check=True)

        # Restore removed .gitkeep
        (static / ".gitkeep").touch()
    except subprocess.CalledProcessError as e:
        print(f"Frontend build failed: {e}", file=sys.stderr)
        raise


class BuildPyCommand(build_py):
    """Build for py."""

    def run(self: Self) -> None:
        """Run."""
        build_frontend()
        super().run()


class DevelopCommand(develop):
    """Build for develop."""

    def run(self: Self) -> None:
        """Run."""
        build_frontend()
        super().run()


class SdistCommand(sdist):
    """Build the sdist."""

    def run(self: Self) -> None:
        """Run."""
        build_frontend()
        super().run()


setup(
    cmdclass={
        "build_py": BuildPyCommand,
        "develop:": DevelopCommand,
        "sdist": SdistCommand,
    },
)
